#define FILEINAME "desk_1280x720.yuv"
#define ONAME "Disparity"

#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include "sds_lib.h"

#include "top.h"
//from top.h
// #define DUAL_WIDTH 1280
// #define IMG_WIDTH  640
// #define IMG_HEIGHT 720

pix_t fileBuf[DUAL_WIDTH];

int read_yuv_file(pix_t *buf, int s, int w, int h, char *fname);
void write_yuv_file(pix_t *buf, int s, int w, int h, char *name);
void convert_Y8toCY16(pix_t *Ybuf, unsigned short *CYbuf, int pixct);
void convert_CY16toY8(yuv_t *CYbuf, pix_t *Ybuf, int pixct);

// monochrome yuv422 file is read using width w * height h
// only Y data is read, and returned in pix_t buffer
// stride s is >= width w
// use buffer pix_t fileBuf[DUAL_WIDTH];
// ignore U and V planes
int read_yuv_file(pix_t *buf, int s, int w, int h, char *fname)
{
    int i,j;
    FILE *f;
    
    f = fopen(fname,"rb");
    if (f==NULL) {
        printf("failed to open file %s\n", fname);
        return -1;
    }
    else {
        printf("read file %s\n", fname);
    }

    // Y
    for (j=0;j<h;j++) {
        fread(fileBuf, w, 1, f);
        for (i=0;i<w;i++) {
            *(buf+j*s+i) = fileBuf[i];
        }
    }

    fclose(f);
    return 0;
}

// monochrome yuv422 file is written from Y data using width w * height h
// w * h is appended to name, recognized by YUV Player Deluxe and other players
// stride s is >= width w
// use buffer pix_t fileBuf[DUAL_WIDTH];
void write_yuv_file(pix_t *buf, int s, int w, int h, char *name)
{
    int i,j;
    char fname[32];
    FILE *f;
    
    sprintf(fname,"%s_%dx%d.yuv",name,w,h);
    f = fopen(fname,"wb");
    if (f==NULL) {
        printf("failed to open file %s\n", fname);
        return;
    }
    else {
        printf("write file %s\n", fname);
    }

    // Y
    for (j=0;j<h;j++) {
        for (i=0;i<w;i++) {
            fileBuf[i] = (*(buf+j*s+i)); // file data is u8
        }
        fwrite(fileBuf, w, 1, f);
    }
    // 4:2:2 U,V each half width as Y
    for (i=0;i<w/2;i++) 
		fileBuf[i] = 0x80;
    for (j=0;j<h;j++) 
        fwrite(fileBuf, w/2, 1, f);
    for (j=0;j<h;j++) 
        fwrite(fileBuf, w/2, 1, f);

    fclose(f);
}

void convert_Y8toCY16(pix_t *Ybuf, yuv_t *CYbuf, int pixct)
{
	int i;
	yuv_t cypix;

	for (i=0; i<pixct; ++i) {
		cypix = 0x8000 | Ybuf[i];
		CYbuf[i] = cypix;
	}
}

void convert_CY16toY8(yuv_t *CYbuf, pix_t *Ybuf, int pixct)
{
	int i;
	pix_t ypix;

	for (i=0; i<pixct; ++i) {
		ypix = (pix_t)(CYbuf[i] & 0x00FF);
		Ybuf[i] = ypix;
	}
}

int main()
{
	pix_t *inY = (pix_t *)sds_alloc(IMG_HEIGHT*DUAL_WIDTH);
	yuv_t *inCY = (yuv_t *)sds_alloc(IMG_HEIGHT*DUAL_WIDTH*2);
	yuv_t *outCY = (yuv_t *)sds_alloc(IMG_HEIGHT*IMG_WIDTH*2);
	pix_t *outY = (pix_t *)sds_alloc(IMG_HEIGHT*IMG_WIDTH);
	printf("allocated buffers\n");

	// read double wide image from disk
	if (read_yuv_file(inY, DUAL_WIDTH, DUAL_WIDTH, IMG_HEIGHT, FILEINAME) != 0)
		return -1;
	convert_Y8toCY16(inY, inCY, IMG_HEIGHT*DUAL_WIDTH);
	printf("converted frame to 16bit\n");

	stereo_remap_bm(
		        inCY, outCY,
		        IMG_HEIGHT, DUAL_WIDTH, DUAL_WIDTH, IMG_WIDTH);
	printf("computed stereo disparity\n");

	// write single wide image to disk
	convert_CY16toY8(outCY, outY, IMG_HEIGHT*IMG_WIDTH);
	printf("converted disparity data to 8bit\n");
	write_yuv_file(outY, IMG_WIDTH, IMG_WIDTH, IMG_HEIGHT, ONAME);

	sds_free(inY);
	sds_free(inCY);
	sds_free(outCY);
	sds_free(outY);
	printf("freed buffers\n");

	printf("TEST PASSED\n");
	return 0;
}
